// @flow
import React, { Component } from 'react';
import './HorizontalCalendar.css';
import CalendarHeader from './CalendarHeader';
import CalendarFooter from './CalendarFooter';
import CalendarContent from './CalendarContent';

class HorizontalCalendar extends Component {
  constructor() {
    super();
    const now = new Date();
    this.state = {
      year: now.getFullYear(),
      month: now.getMonth(),
      selectedDay: now.getDate(),
    };
  }

  state: {
    year: number,
    month: number,
    selectedDay: number,
  };

  /**
   * 获取每个月份的天数
   */
  getMonthDays() {
    const temp = new Date(this.state.year, this.state.month + 1, 0);
    return temp.getDate();
  }

  /**
   * 获取每个月第一天是周几
   */
  getFirstDayWeek() {
    const dt = new Date(`${this.state.year}/${this.state.month + 1}/1`);
    const Weekdays = dt.getDay();
    return Weekdays;
  }

  /**
   * 获取当月应展示的数据
   */
  getDays() {
    const arr1 = [];
    const arr2 = [];
    const getDayItems = this.getMonthDays();
    const firstDayWeek = this.getFirstDayWeek();
    for (let i = 0; i < firstDayWeek; i++) {
      arr1[i] = -1;
    }
    for (let i = 0; i < getDayItems; i++) {
      arr2[i] = i + 1;
    }
    return arr1.concat(arr2);
  }

  /**
   * 切换到下一月
   */
  nextMonth() {
    if (this.state.month === 11) {
      this.setState({
        year: this.state.year + 1,
        month: 0,
      });
    } else {
      this.setState({
        month: this.state.month + 1,
      });
    }
  }

  /**
   * 切换到上一月
   */
  prevMonth() {
    if (this.state.month === 0) {
      this.setState({
        year: this.state.year - 1,
        month: 11,
      });
    } else {
      this.setState({
        month: this.state.month - 1,
      });
    }
  }
  handleDaySelected(selectedDay: number) {
    this.setState({
      selectedDay,
    });
  }

  render() {
    return (
      <div className="sinooa-horizontal-calendar">
        <CalendarHeader
          prevMonth={() => this.prevMonth()}
          nextMonth={() => this.nextMonth()}
          year={this.state.year}
          month={this.state.month}
        />
        <div className="sinooa-horizontal-calendar-content">
          <div className="sinooa-horizontal-calendar-left">
            <CalendarContent
              days={this.getDays()}
              selectedDay={this.state.selectedDay}
              onSelected={value => this.handleDaySelected(value)}
            />
          </div>
          <CalendarFooter
            month={this.state.month}
            selectedDay={this.state.selectedDay}
          />
        </div>
      </div>
    );
  }
}

export default HorizontalCalendar;
