import React from 'react';
import { DragSource } from 'react-dnd';
import classNames from 'classnames';
import './IndexCell.css';

type IndexCellPropsType = {
  flex: number,
  width: string,
  children: any,
  id: string,
  index: number,
  isDragging: boolean,
  connectDragSource: () => void,
  connectDropTarget: () => void,
};

const cellSource = {
  beginDrag(props) {
    return {
      id: props.id,
      column: props.column,
    };
  },
};

function IndexCell(props: IndexCellPropsType) {
  const content = (
    <div
      className={classNames('sinooa-index-cell', props.className)}
      style={{
        width: props.width,
      }}
    >
      {props.children}
    </div>
  );
  return props.canDrag ? props.connectDragSource(content) : content;
}

export default DragSource(
  'WorkbenchLayout',
  cellSource,
  (connect, monitor) => ({
    connectDragSource: connect.dragSource(),
    isDragging: monitor.isDragging(),
  }),
)(IndexCell);
