import React from 'react';
import { DropTarget } from 'react-dnd';
import { connect as reduxConnect } from 'react-redux';
import './Workbench.css';
import IndexColumn from './IndexColumn';
import IndexCell from './IndexCell';
import IndexLayout from '../modal/IndexLayout';
import IndexLayoutModal from '../modal/IndexLayoutModal';
import config from '../config';
import { ActionCreators } from '../actions';
import { layout1 } from '../data';

type RegionPropsType = {
  color: string,
  height: number,
};

function Region(props: RegionPropsType) {
  return (
    <div
      className="sinooa-region"
      style={{
        backgroundColor: props.color,
        height: props.height || 250,
      }}
    />
  );
}

class Workbench extends React.Component {
  static defaultProps = {
    layout: layout1,
  };
  constructor(props) {
    super(props);
    this.state = {
      data: [],
      movingCellId: undefined,
    };
    this.indexLayoutData = {};

    this.onMoveToColumn = this.onMoveToColumn.bind(this);
  }

  componentWillMount() {
    this.indexLayoutModal = new IndexLayoutModal(this.props.cells);
    this.indexLayoutModal.setLayout(this.props.layout);
    this.setState({ data: this.indexLayoutModal.modal });
  }

  componentDidMount() {
    this.width = this.rootDom.clientWidth;
    this.offsetLeft = this.rootDom.offsetLeft;
    this.offsetTop = this.rootDom.offsetTop;
    const layout = this.state.data.map(column => ({
      ...column,
      cells: column.cells.map(cell => cell.id),
    }));
    this.props.dispatch(ActionCreators.saveLayout(layout));
  }

  onMoveToColumn(cellId, point) {
    const indexLayout = new IndexLayout(
      this.width,
      this.indexLayoutModal.modal,
    );
    const x = point.x - this.offsetLeft;
    const y = point.y - this.offsetTop;
    const relativePoint = indexLayout.getRelativePoint(x, y);
    this.indexLayoutModal.moveCell(cellId, relativePoint);

    this.setState({ movingCellId: cellId });

    if (this.state.data !== this.indexLayoutModal.modal) {
      this.setState({
        data: this.indexLayoutModal.modal,
      });
    }

    const layout = this.state.data.map(column => ({
      ...column,
      cells: column.cells.map(cell => cell.id),
    }));

    this.props.changeLayout && this.props.changeLayout(layout);
    this.props.dispatch(ActionCreators.changeLayout(layout));
  }

  props: {
    cells: Array[],
    layout: Array[],
    canDrag: boolean,
    connectDropTarget: () => void,
    changeLayout: () => void,
    dispatch: () => void,
  };
  indexLayoutModal: IndexLayoutModal;

  renderCell = (column, cell) => {
    if (config[cell.id]) {
      return config[cell.id].render(column, cell);
    }
    return <Region color={cell.color} height={cell.height} />;
  };

  render() {
    return this.props.connectDropTarget(
      <div
        className="sinooa-index-layout"
        ref={dom => {
          this.rootDom = dom;
        }}
      >
        {this.state.data.map(column => (
          <IndexColumn
            id={column.id}
            key={column.id}
            flex={column.flex}
            move={this.onMoveToColumn}
          >
            {column.cells.map(cell => (
              <IndexCell
                className={
                  this.state.movingCellId === cell.id
                    ? 'sinooa-moving-cell'
                    : ''
                }
                flex={cell.flex}
                id={cell.id}
                index={cell.index}
                key={cell.id}
                column={column.id}
                canDrag={this.props.canDrag}
              >
                {this.renderCell(column, cell)}
              </IndexCell>
            ))}
          </IndexColumn>
        ))}
      </div>,
    );
  }
}

const cellTarget = {
  hover(props, monitor, component) {
    component.onMoveToColumn(monitor.getItem().id, monitor.getClientOffset());
  },

  drop(props, monitor, component) {
    component.setState({ movingCellId: undefined });
  },
};

const mapDispatchToProps = dispatch => ({
  dispatch,
});

export default reduxConnect(null, mapDispatchToProps)(
  DropTarget('WorkbenchLayout', cellTarget, connect => ({
    connectDropTarget: connect.dropTarget(),
  }))(Workbench),
);
