import ColumnLayout from './ColumnLayout';

describe('测试区域布局', () => {
  const flex = 6;
  const width = 600;
  const cells = [
    {
      id: 'cell1',
      flex: 3,
      color: 'red',
      height: 200,
    },
    {
      id: 'cell2',
      flex: 3,
      color: 'yellow',
      height: 200,
    },
    {
      id: 'cell3',
      color: 'green',
      height: 250,
    },
    {
      id: 'cell4',
      color: 'black',
      flex: 3,
      height: 250,
    },
    {
      id: 'cell5',
      color: 'grey',
      height: 250,
    },
  ];

  const rows = [
    {
      cells: [
        { color: 'red', flex: 3, height: 200, id: 'cell1' },
        {
          color: 'yellow',
          flex: 3,
          height: 200,
          id: 'cell2',
        },
      ],
      flex: 6,
      height: 200,
      width: 600,
      y: 0,
    },
    {
      cells: [{ color: 'green', height: 250, id: 'cell3' }],
      flex: 6,
      height: 250,
      width: 600,
      y: 200,
    },
    {
      cells: [
        {
          color: 'black',
          flex: 3,
          height: 250,
          id: 'cell4',
        },
      ],
      flex: 6,
      height: 250,
      width: 600,
      y: 450,
    },
    {
      cells: [{ color: 'grey', height: 250, id: 'cell5' }],
      flex: 6,
      height: 250,
      width: 600,
      y: 700,
    },
  ];

  it('模型创建,返回指定元素', () => {
    const columnLayout = new ColumnLayout(flex, width, cells);

    expect(columnLayout.flex).toEqual(6);
    expect(columnLayout.width).toEqual(600);
    expect(columnLayout.rows).toEqual(rows);
  });

  it('获取当前行的信息', () => {
    const columnLayout = new ColumnLayout(flex, width, cells);
    const currentRows = columnLayout.getCurrentRow();

    expect(currentRows.cells).toEqual([
      { color: 'grey', height: 250, id: 'cell5' },
    ]);
    expect(currentRows.width).toEqual(600);
    expect(currentRows.flex).toEqual(6);
    expect(currentRows.height).toEqual(250);
    expect(currentRows.y).toEqual(700);
  });

  it('获取区块布局信息', () => {
    const columnLayout = new ColumnLayout(flex, width, cells);

    expect(columnLayout.rows).toEqual(rows);
  });

  it('坐标为(100, 240)的点位于区块2,3之间', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 100,
      y: 240,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: 'cell2', next: 'cell3' });
  });

  it('坐标为(60, 600)的点位于区块3,4之间', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 60,
      y: 600,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: 'cell3', next: 'cell4' });
  });

  it('坐标为(200, 600)的点位于区块4,5之间', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 200,
      y: 600,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: 'cell4', next: 'cell5' });
  });

  it('坐标为(400, 800)的点位于区块5之后', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 400,
      y: 800,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: 'cell5', next: null });
  });

  it('坐标为(50, 100)的点位于区块1之前', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 50,
      y: 100,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: null, next: 'cell1' });
  });

  it('坐标为(200, 100)的点位于区块1,2之间', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 200,
      y: 100,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: 'cell1', next: 'cell2' });
  });

  it('坐标(450,100)的点应该位于区块1,2之间', () => {
    const columnLayout = new ColumnLayout(flex, width, cells, {
      x: 450,
      y: 100,
    });
    const pointPosition = columnLayout.getPointPosition();

    expect(pointPosition).toEqual({ before: 'cell1', next: 'cell2' });
  });
});
