import ColumnLayout from './ColumnLayout';

/**
 * 首页布局服务
 *
 * @export
 * @class IndexLayout
 */
export default class IndexLayout {
  columnPos: number[];
  columnWidths: number[];

  /**
   * Creates an instance of IndexLayout.
   * @param {any} screenWidth 屏幕宽度
   * @param {any} columns  区域信息
   * @memberof IndexLayout
   */
  constructor(screenWidth, columns) {
    this.screenWidth = screenWidth;
    this.columns = columns;

    this.calcColumnPos();
  }

  calcColumnPos() {
    let pos = 0;
    this.columnPos = [];
    this.columnWidths = [];
    const flexWidth = this.screenWidth / 12;
    for (const column of this.columns) {
      const width = column.flex * flexWidth;
      pos += width;
      this.columnWidths.push(width);
      this.columnPos.push(pos);
    }
  }

  /**
   * 获取坐标所在的区域
   *
   * @returns
   * @memberof IndexLayout
   */
  getColumn(x, y) {
    const columnPos = this.columnPos;

    return columnPos.findIndex(pos => x <= pos);
  }

  /**
   * 获取坐标的相对位置
   *
   * @param {any} x
   * @param {any} y
   * @returns
   * @memberof IndexLayout
   */
  getRelativePoint(x, y) {
    const columnIndex = this.getColumn(x, y);

    if (columnIndex === -1) {
      return undefined;
    }

    const column = this.columns[columnIndex];
    const width = this.columnWidths[columnIndex];
    const columnLayout = new ColumnLayout(column.flex, width, column.cells, {
      x: x - (columnIndex === 0 ? 0 : this.columnPos[columnIndex - 1]),
      y,
    });

    return {
      ...columnLayout.getPointPosition(),
      column: column.id,
    };
  }
}
