import IndexLayoutModal from './IndexLayoutModal';

describe('首页布局模型数据', () => {
  const cells = [
    {
      id: '1',
      flex: 3,
      height: 200,
    },
    {
      id: '2',
      flex: 3,
      height: 200,
    },
    {
      id: '3',
      height: 250,
    },
    {
      id: '4',
      flex: 3,
      height: 250,
    },
    {
      id: '5',
      height: 250,
    },
    {
      id: '6',
      flex: 3,
      height: 250,
    },
    {
      id: '7',
      height: 250,
    },
    {
      id: '8',
      flex: 3,
      height: 250,
    },
    {
      id: '9',
      height: 250,
    },
  ];

  it('构建首页布局模型数据', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    expect(indexLayoutModal.cells).toEqual(cells);
    expect(indexLayoutModal.columns).toEqual([]);
  });

  it('已知区域信息，获取区域中的所有模块', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);

    expect(indexLayoutModal.columns[0].flex).toEqual(6);
    expect(indexLayoutModal.columns[0].id).toEqual('1');
    expect(indexLayoutModal.columns[0].cells).toEqual([
      {
        id: '1',
        flex: 3,
        height: 200,
      },
      {
        id: '2',
        flex: 3,
        height: 200,
      },
      {
        id: '3',
        height: 250,
      },
    ]);
  });

  it('给定区块5及其相对位置，重新构建模型', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);
    indexLayoutModal.moveCell('5', { before: '1', next: '2', column: '1' });

    expect(indexLayoutModal.columns[0].flex).toEqual(6);
    expect(indexLayoutModal.columns[0].id).toEqual('1');
    expect(indexLayoutModal.columns[0].cells).toEqual([
      { flex: 3, height: 200, id: '1' },
      { height: 250, id: '5' },
      { flex: 3, height: 200, id: '2' },
      { height: 250, id: '3' },
    ]);
  });

  it('获取模型数据', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    expect(indexLayoutModal.modal).toEqual([]);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);

    expect(indexLayoutModal.modal).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '1' },
          { flex: 3, height: 200, id: '2' },
          { height: 250, id: '3' },
        ],
        flex: 6,
        id: '1',
      },
    ]);
  });

  it('将区块5插入区块1之前，重新构建数据模型', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1'] }]);
    indexLayoutModal.moveCell('5', { before: null, next: '1', column: '1' });

    expect(indexLayoutModal.columns[0].cells).toEqual([
      { height: 250, id: '5' },
      { flex: 3, height: 200, id: '1' },
    ]);
  });

  it('将区块5插入区块2和区块3之间，重新构建数据模型', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);
    indexLayoutModal.moveCell('5', { before: '2', next: '3', column: '1' });

    expect(indexLayoutModal.columns[0].cells).toEqual([
      { flex: 3, height: 200, id: '1' },
      { flex: 3, height: 200, id: '2' },
      { height: 250, id: '5' },
      { height: 250, id: '3' },
    ]);
  });

  it('将区块1移动到区块2和区块3之间', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);
    indexLayoutModal.moveCell('1', { before: '2', next: '3', column: '1' });

    expect(indexLayoutModal.columns[0].cells).toEqual([
      { flex: 3, height: 200, id: '2' },
      { flex: 3, height: 200, id: '1' },
      { height: 250, id: '3' },
    ]);
  });

  it('将区块2移动到区块3之后', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);
    indexLayoutModal.moveCell('2', { before: '3', next: null, column: '1' });

    expect(indexLayoutModal.columns[0].cells).toEqual([
      { flex: 3, height: 200, id: '1' },
      { height: 250, id: '3' },
      { flex: 3, height: 200, id: '2' },
    ]);
  });
  it('将区块2移动到区块1之前', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([{ flex: 6, cells: ['1', '2', '3'] }]);
    indexLayoutModal.moveCell('2', { before: null, next: '1', column: '1' });

    expect(indexLayoutModal.columns[0].cells).toEqual([
      { flex: 3, height: 200, id: '2' },
      { flex: 3, height: 200, id: '1' },
      { height: 250, id: '3' },
    ]);
  });

  it('指定两个区块信息，构建模型', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([
      { flex: 6, cells: ['2', '4'] },
      { flex: 3, cells: ['5', '6'] },
    ]);

    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      {
        cells: [{ height: 250, id: '5' }, { flex: 3, height: 250, id: '6' }],
        flex: 3,
        id: '2',
      },
    ]);
  });

  it('将区块3放入区域1并且放入区块4和区块5之间', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([
      { flex: 6, cells: ['2', '4'] },
      { flex: 3, cells: ['5', '6'] },
    ]);

    indexLayoutModal.moveCell('3', { before: '4', next: '5', column: '1' });

    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
          { height: 250, id: '3' },
        ],
        flex: 6,
        id: '1',
      },
      {
        cells: [{ height: 250, id: '5' }, { flex: 3, height: 250, id: '6' }],
        flex: 3,
        id: '2',
      },
    ]);
  });

  it('将区块3放入区域2并且放入区块4和区块5之间', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([
      { flex: 6, cells: ['2', '4'] },
      { flex: 3, cells: ['5', '6'] },
    ]);

    indexLayoutModal.moveCell('3', { before: '4', next: '5', column: '2' });

    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      {
        cells: [
          { height: 250, id: '3' },
          { height: 250, id: '5' },
          { flex: 3, height: 250, id: '6' },
        ],
        flex: 3,
        id: '2',
      },
    ]);
  });

  it('将区块多次移动，构建数据模型', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([
      { flex: 6, cells: ['2', '4'] },
      { flex: 3, cells: ['5', '6'] },
    ]);

    indexLayoutModal.moveCell('5', { before: '2', next: '4', column: '1' });

    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { height: 250, id: '5' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      { cells: [{ flex: 3, height: 250, id: '6' }], flex: 3, id: '2' },
    ]);

    indexLayoutModal.moveCell('5', { before: null, next: '2', column: '1' });
    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { height: 250, id: '5' },
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      { cells: [{ flex: 3, height: 250, id: '6' }], flex: 3, id: '2' },
    ]);

    indexLayoutModal.moveCell('5', { before: '6', next: null, column: '2' });
    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      {
        cells: [{ flex: 3, height: 250, id: '6' }, { height: 250, id: '5' }],
        flex: 3,
        id: '2',
      },
    ]);

    indexLayoutModal.moveCell('2', { before: '2', next: null, column: '1' });
    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      {
        cells: [{ flex: 3, height: 250, id: '6' }, { height: 250, id: '5' }],
        flex: 3,
        id: '2',
      },
    ]);

    indexLayoutModal.moveCell('5', { before: '5', next: null, column: '2' });
    expect(indexLayoutModal.columns).toEqual([
      {
        cells: [
          { flex: 3, height: 200, id: '2' },
          { flex: 3, height: 250, id: '4' },
        ],
        flex: 6,
        id: '1',
      },
      {
        cells: [{ flex: 3, height: 250, id: '6' }, { height: 250, id: '5' }],
        flex: 3,
        id: '2',
      },
    ]);
  });

  it('多次布局,重新构建数据模型', () => {
    const indexLayoutModal = new IndexLayoutModal(cells);

    indexLayoutModal.setLayout([
      { flex: 6, cells: ['2', '4'] },
      { flex: 3, cells: ['5', '6'] },
      { flex: 3, cells: [] },
    ]);

    expect(indexLayoutModal.columns[0].flex).toEqual(6);
    expect(indexLayoutModal.columns[1].flex).toEqual(3);
    expect(indexLayoutModal.columns[2].flex).toEqual(3);
    expect(indexLayoutModal.columns[0].cells).toEqual([
      { flex: 3, height: 200, id: '2' },
      { flex: 3, height: 250, id: '4' },
    ]);
    expect(indexLayoutModal.columns[1].cells).toEqual([
      { height: 250, id: '5' },
      { flex: 3, height: 250, id: '6' },
    ]);
    expect(indexLayoutModal.columns[2].cells).toEqual([]);

    indexLayoutModal.setLayout([
      { flex: 3, cells: [] },
      { flex: 6, cells: ['2'] },
      { flex: 3, cells: ['1'] },
    ]);

    expect(indexLayoutModal.columns[0].flex).toEqual(3);
    expect(indexLayoutModal.columns[1].flex).toEqual(6);
    expect(indexLayoutModal.columns[2].flex).toEqual(3);
    expect(indexLayoutModal.columns[0].cells).toEqual([]);
    expect(indexLayoutModal.columns[1].cells).toEqual([
      { flex: 3, height: 200, id: '2' },
    ]);
    expect(indexLayoutModal.columns[2].cells).toEqual([
      { flex: 3, height: 200, id: '1' },
    ]);
  });
});
