import config from '../config';

/**
 * 获取剩余空间
 *
 * @param {number} width
 * @param {number} flex
 * @param {*} cells
 * @returns
 */
function getRemainder(width: number, flex: number, cells: any) {
  let occupanciedWidth = 0;
  for (let i = 0; i < cells.length; i++) {
    if (cells[i].flex) {
      occupanciedWidth += cells[i].flex * (width / flex);
    } else {
      occupanciedWidth = width;
    }
  }
  const remainderWidth = width - occupanciedWidth;
  return remainderWidth / (width / flex);
}

/**
 * 区域分行服务
 *
 * @export
 * @class Row
 */
export default class Row {
  /**
   * Creates an instance of Row.
   * @param {number} width 区域宽度
   * @param {number} flex 区域所占比例
   * @param {number} y 区域的开始位置的纵坐标
   * @memberof Row
   */
  constructor(width: number, flex: number, y: number) {
    this.width = width;
    this.flex = flex;
    this.y = y;
    this.cells = [];
    this.height = 0;
  }

  /**
   * 判断是否可以容纳区块
   *
   * @param {any} cell
   * @returns
   * @memberof Row
   */
  canHold(cell) {
    if (this.cells.length === 0) {
      return true;
    }
    const rowRemainder = getRemainder(this.width, this.flex, this.cells);
    const flex = cell.flex ? cell.flex : this.flex;
    if (flex <= rowRemainder) {
      return true;
    }
    return false;
  }

  /**
   * 添加区块
   *
   * @param {any} cell
   * @memberof Row
   */
  addCell(cell) {
    this.cells.push(cell);
    const getHeight = config[cell.id].getHeight;
    let height = cell.height;
    if (getHeight) {
      height = getHeight(
        {
          flex: this.flex,
        },
        cell,
      );
    }
    this.height = Math.max(this.height, height);
  }
}
