import Row from './Row';

describe('测试行元素', () => {
  const width = 600;
  const flex = 6;
  const y = 0;

  const cell1 = {
    id: 'cell1',
    flex: 3,
    color: 'red',
    height: 200,
    parent: 'column1',
  };

  const cell2 = {
    id: 'cell2',
    flex: 3,
    color: 'red',
    height: 250,
    parent: 'column1',
  };

  let row;

  beforeEach(() => {
    row = new Row(width, flex, y);
  });

  it('创建row模型,获取指定值', () => {
    expect(row.cells).toEqual([]);
    expect(row.width).toEqual(600);
    expect(row.flex).toEqual(6);
    expect(row.y).toEqual(0);
  });

  it('row模型中能否容纳当前区块', () => {
    expect(
      row.canHold({
        flex: 3,
      }),
    ).toBeTruthy();
    expect(
      row.canHold({
        flex: 10,
      }),
    ).toBeTruthy();

    row.addCell({
      flex: 3,
    });

    expect(
      row.canHold({
        flex: 3,
      }),
    ).toBeTruthy();

    expect(
      row.canHold({
        flex: 6,
      }),
    ).toBeFalsy();

    expect(row.canHold({})).toBeFalsy();
  });

  it('添加一个自适应宽度cell后，无法再容纳其他cell', () => {
    row.addCell({});

    expect(
      row.canHold({
        flex: 1,
      }),
    ).toBeFalsy();

    expect(row.canHold({})).toBeFalsy();
  });

  it('测试添加区块的方法', () => {
    row.addCell(cell1);
    expect(row.cells).toEqual([cell1]);
    expect(row.height).toEqual(200);

    row.addCell(cell2);
    expect(row.cells).toEqual([cell1, cell2]);
    expect(row.height).toEqual(250);
  });
});
